@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: Apache-2.0
 // https://github.com/AcademySoftwareFoundation/OpenImageIO
 
+#include <OpenImageIO/color.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imageio.h>
@@ -31,6 +32,35 @@
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
+
+class HeifReader final : public heif::Context::Reader {
+public:
+    HeifReader(Filesystem::IOProxy* ioproxy)
+        : m_ioproxy(ioproxy)
+    {
+        m_ioproxy->seek(0);
+    }
+    int64_t get_position() const override { return m_ioproxy->tell(); }
+    int read(void* data, size_t size) override
+    {
+        return m_ioproxy->read(data, size) == size ? 0 : -1;
+    }
+    int seek(int64_t position) override
+    {
+        return m_ioproxy->seek(position) ? 0 : -1;
+    }
+    heif_reader_grow_status wait_for_file_size(int64_t target_size) override
+    {
+        return target_size <= int64_t(m_ioproxy->size())
+                   ? heif_reader_grow_status_size_reached
+                   : heif_reader_grow_status_size_beyond_eof;
+    }
+
+private:
+    Filesystem::IOProxy* m_ioproxy;
+};
+
+
 class HeifInput final : public ImageInput {
 public:
     HeifInput() {}
@@ -38,13 +68,13 @@ public:
     const char* format_name(void) const override { return "heif"; }
     int supports(string_view feature) const override
     {
-        return feature == "exif"
+        return feature == "exif" || feature == "ioproxy"
 #if LIBHEIF_HAVE_VERSION(1, 9, 0)
                || feature == "cicp"
 #endif
             ;
     }
-    bool valid_file(const std::string& filename) const override;
+    bool valid_file(Filesystem::IOProxy* ioproxy) const override;
     bool open(const std::string& name, ImageSpec& newspec) override;
     bool open(const std::string& name, ImageSpec& newspec,
               const ImageSpec& config) override;
@@ -66,6 +96,7 @@ private:
     bool m_do_associate            = false;
     bool m_reorient                = true;
     std::unique_ptr<heif::Context> m_ctx;
+    std::unique_ptr<HeifReader> m_reader;
     heif_item_id m_primary_id;             // id of primary image
     std::vector<heif_item_id> m_item_ids;  // ids of all other images
     heif::ImageHandle m_ihandle;
@@ -73,7 +104,6 @@ private:
 };
 
 
-
 void
 oiio_heif_init()
 {
@@ -110,10 +140,12 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 bool
-HeifInput::valid_file(const std::string& filename) const
+HeifInput::valid_file(Filesystem::IOProxy* ioproxy) const
 {
+    if (!ioproxy || ioproxy->mode() != Filesystem::IOProxy::Mode::Read)
+        return false;
     uint8_t magic[12];
-    if (Filesystem::read_bytes(filename, magic, sizeof(magic)) != sizeof(magic))
+    if (ioproxy->pread(magic, sizeof(magic), 0) != sizeof(magic))
         return false;
     heif_filetype_result filetype_check = heif_check_filetype(magic,
                                                               sizeof(magic));
@@ -140,7 +172,12 @@ HeifInput::open(const std::string& name, ImageSpec& newspec,
     m_filename = name;
     m_subimage = -1;
 
+    ioproxy_retrieve_from_config(config);
+    if (!ioproxy_use_or_open(name))
+        return false;
+
     m_ctx.reset(new heif::Context);
+    m_reader.reset(new HeifReader(ioproxy()));
     m_himage  = heif::Image();
     m_ihandle = heif::ImageHandle();
 
@@ -149,8 +186,7 @@ HeifInput::open(const std::string& name, ImageSpec& newspec,
     m_reorient = config.get_int_attribute("oiio:reorient", 1);
 
     try {
-        m_ctx->read_from_file(name);
-        // FIXME: should someday be read_from_reader to give full flexibility
+        m_ctx->read_from_reader(*m_reader);
 
         m_item_ids   = m_ctx->get_list_of_top_level_image_IDs();
         m_primary_id = m_ctx->get_primary_image_ID();
@@ -186,6 +222,7 @@ HeifInput::close()
     m_himage  = heif::Image();
     m_ihandle = heif::ImageHandle();
     m_ctx.reset();
+    m_reader.reset();
     m_subimage                = -1;
     m_num_subimages           = 0;
     m_associated_alpha        = true;
@@ -225,15 +262,40 @@ HeifInput::seek_subimage(int subimage, int miplevel)
     }
 
     m_has_alpha = m_ihandle.has_alpha_channel();
-    auto chroma = m_has_alpha        ? (m_bitdepth > 8)
-                                           ? littleendian()
-                                                 ? heif_chroma_interleaved_RRGGBBAA_LE
-                                                 : heif_chroma_interleaved_RRGGBBAA_BE
-                                           : heif_chroma_interleaved_RGBA
-                  : (m_bitdepth > 8) ? littleendian()
-                                           ? heif_chroma_interleaved_RRGGBB_LE
-                                           : heif_chroma_interleaved_RRGGBB_BE
-                                     : heif_chroma_interleaved_RGB;
+
+    bool is_monochrome = false;
+
+#if LIBHEIF_NUMERIC_VERSION >= MAKE_LIBHEIF_VERSION(1, 17, 0, 0)
+    heif_colorspace preferred_colorspace = heif_colorspace_undefined;
+    heif_chroma preferred_chroma         = heif_chroma_undefined;
+
+    if (heif_image_handle_get_preferred_decoding_colorspace(
+            m_ihandle.get_raw_image_handle(), &preferred_colorspace,
+            &preferred_chroma)
+            .code
+        == heif_error_Ok) {
+        is_monochrome = preferred_colorspace == heif_colorspace_monochrome;
+    }
+#endif
+
+    const heif_chroma chroma
+        = (is_monochrome)    ? heif_chroma_monochrome
+          : m_has_alpha      ? (m_bitdepth > 8)
+                                   ? littleendian()
+                                         ? heif_chroma_interleaved_RRGGBBAA_LE
+                                         : heif_chroma_interleaved_RRGGBBAA_BE
+                                   : heif_chroma_interleaved_RGBA
+          : (m_bitdepth > 8) ? littleendian()
+                                   ? heif_chroma_interleaved_RRGGBB_LE
+                                   : heif_chroma_interleaved_RRGGBB_BE
+                             : heif_chroma_interleaved_RGB;
+    const heif_colorspace colorspace = is_monochrome
+                                           ? heif_colorspace_monochrome
+                                           : heif_colorspace_RGB;
+    const heif_channel channel       = is_monochrome ? heif_channel_Y
+                                                     : heif_channel_interleaved;
+    const int nchannels              = is_monochrome ? 1 : m_has_alpha ? 4 : 3;
+
 #if 0
     try {
         m_himage = m_ihandle.decode_image(heif_colorspace_RGB, chroma);
@@ -253,8 +315,8 @@ HeifInput::seek_subimage(int subimage, int miplevel)
     // print("Got decoding options version {}\n", options->version);
     struct heif_image* img_tmp = nullptr;
     struct heif_error herr = heif_decode_image(m_ihandle.get_raw_image_handle(),
-                                               &img_tmp, heif_colorspace_RGB,
-                                               chroma, options.get());
+                                               &img_tmp, colorspace, chroma,
+                                               options.get());
     if (img_tmp)
         m_himage = heif::Image(img_tmp);
     if (herr.code != heif_error_Ok || !img_tmp) {
@@ -264,9 +326,8 @@ HeifInput::seek_subimage(int subimage, int miplevel)
     }
 #endif
 
-    m_spec = ImageSpec(m_himage.get_width(heif_channel_interleaved),
-                       m_himage.get_height(heif_channel_interleaved),
-                       m_has_alpha ? 4 : 3,
+    m_spec = ImageSpec(m_himage.get_width(channel),
+                       m_himage.get_height(channel), nchannels,
                        (m_bitdepth > 8) ? TypeUInt16 : TypeUInt8);
 
     if (m_bitdepth > 8) {
@@ -286,12 +347,24 @@ HeifInput::seek_subimage(int subimage, int miplevel)
             m_ihandle.get_raw_image_handle(), &nclx);
 
         if (nclx) {
-            if (err.code == heif_error_Ok) {
+            // When CICP metadata is not present in the file, libheif returns
+            // unspecified since v1.21. Ignore it then.
+            if (err.code == heif_error_Ok
+                && !(nclx->color_primaries == heif_color_primaries_unspecified
+                     && nclx->transfer_characteristics
+                            == heif_transfer_characteristic_unspecified
+                     && nclx->matrix_coefficients
+                            == heif_matrix_coefficients_unspecified)) {
                 const int cicp[4] = { int(nclx->color_primaries),
                                       int(nclx->transfer_characteristics),
                                       int(nclx->matrix_coefficients),
                                       int(nclx->full_range_flag ? 1 : 0) };
                 m_spec.attribute("CICP", TypeDesc(TypeDesc::INT, 4), cicp);
+                const ColorConfig& colorconfig(
+                    ColorConfig::default_colorconfig());
+                string_view interop_id = colorconfig.get_color_interop_id(cicp);
+                if (!interop_id.empty())
+                    m_spec.attribute("oiio:ColorSpace", interop_id);
             }
             heif_nclx_color_profile_free(nclx);
         }
@@ -443,12 +516,13 @@ HeifInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
 #else
     int ystride = 0;
 #endif
+    const heif_channel channel = m_spec.nchannels == 1
+                                     ? heif_channel_Y
+                                     : heif_channel_interleaved;
 #if LIBHEIF_NUMERIC_VERSION >= MAKE_LIBHEIF_VERSION(1, 20, 2, 0)
-    const uint8_t* hdata = m_himage.get_plane2(heif_channel_interleaved,
-                                               &ystride);
+    const uint8_t* hdata = m_himage.get_plane2(channel, &ystride);
 #else
-    const uint8_t* hdata = m_himage.get_plane(heif_channel_interleaved,
-                                              &ystride);
+    const uint8_t* hdata = m_himage.get_plane(channel, &ystride);
 #endif
     if (!hdata) {
         errorfmt("Unknown read error");
