@@ -226,12 +226,7 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
     if (png_get_sRGB(sp, ip, &srgb_intent)) {
         spec.attribute("oiio:ColorSpace", "srgb_rec709_scene");
     } else if (png_get_gAMA(sp, ip, &gamma) && gamma > 0.0) {
-        // Round gamma to the nearest hundredth to prevent stupid
-        // precision choices and make it easier for apps to make
-        // decisions based on known gamma values. For example, you want
-        // 2.2, not 2.19998.
         float g = float(1.0 / gamma);
-        g       = roundf(100.0f * g) / 100.0f;
         set_colorspace_rec709_gamma(spec, g);
     } else {
         // If there's no info at all, assume sRGB.
@@ -330,8 +325,12 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
     {
         png_byte pri = 0, trc = 0, mtx = 0, vfr = 0;
         if (png_get_cICP(sp, ip, &pri, &trc, &mtx, &vfr)) {
-            int cicp[4] = { pri, trc, mtx, vfr };
+            const int cicp[4] = { pri, trc, mtx, vfr };
             spec.attribute(CICP_ATTR, TypeDesc(TypeDesc::INT, 4), cicp);
+            const ColorConfig& colorconfig(ColorConfig::default_colorconfig());
+            string_view interop_id = colorconfig.get_color_interop_id(cicp);
+            if (!interop_id.empty())
+                spec.attribute("oiio:ColorSpace", interop_id);
         }
     }
 #endif
@@ -564,12 +563,30 @@ put_parameter(png_structp& sp, png_infop& ip, const std::string& _name,
         return true;
     }
 #endif
+
+    // Before handling general named metadata, suppress format-specific
+    // metadata hints meant for other formats that are not meant to be literal
+    // metadata written to the file. This includes anything with a namespace
+    // prefix of "oiio:" or the name of any other file format.
+    auto colonpos = name.find(':');
+    if (colonpos != std::string::npos) {
+        std::string prefix = Strutil::lower(name.substr(0, colonpos));
+        if (prefix != "png" && is_imageio_format_name(prefix))
+            return false;
+        if (prefix == "oiio")
+            return false;
+    }
+
     if (type == TypeDesc::STRING) {
+        // We can save arbitrary string metadata in multiple png text entries.
+        // Is that ok? Should we also do it for other types by converting to
+        // string?
         png_text t;
         t.compression = PNG_TEXT_COMPRESSION_NONE;
         t.key         = (char*)ustring(name).c_str();
         t.text        = *(char**)data;  // Already uniquified
         text.push_back(t);
+        return true;
     }
 
     return false;
@@ -608,7 +625,8 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
     string_view colorspace = spec.get_string_attribute("oiio:ColorSpace",
                                                        "srgb_rec709_scene");
     const ColorConfig& colorconfig(ColorConfig::default_colorconfig());
-    srgb = false;
+    OIIO_MAYBE_UNUSED bool wrote_colorspace = false;
+    srgb                                    = false;
     if (colorconfig.equivalent(colorspace, "srgb_rec709_scene")) {
         srgb  = true;
         gamma = 1.0f;
@@ -628,7 +646,8 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA chunk";
         png_set_gAMA(sp, ip, 1.0);
-        srgb = false;
+        srgb             = false;
+        wrote_colorspace = true;
     } else if (Strutil::istarts_with(colorspace, "Gamma")) {
         // Back compatible, this is DEPRECATED(3.1)
         Strutil::parse_word(colorspace);
@@ -638,24 +657,28 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA chunk";
         png_set_gAMA(sp, ip, 1.0f / gamma);
-        srgb = false;
+        srgb             = false;
+        wrote_colorspace = true;
     } else if (colorconfig.equivalent(colorspace, "g22_rec709_scene")) {
         gamma = 2.2f;
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA chunk";
         png_set_gAMA(sp, ip, 1.0f / gamma);
-        srgb = false;
+        srgb             = false;
+        wrote_colorspace = true;
     } else if (colorconfig.equivalent(colorspace, "g18_rec709_scene")) {
         gamma = 1.8f;
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA chunk";
         png_set_gAMA(sp, ip, 1.0f / gamma);
-        srgb = false;
+        srgb             = false;
+        wrote_colorspace = true;
     } else if (colorconfig.equivalent(colorspace, "srgb_rec709_scene")) {
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG gAMA and cHRM chunk";
         png_set_sRGB_gAMA_and_cHRM(sp, ip, PNG_sRGB_INTENT_ABSOLUTE);
-        srgb = true;
+        srgb             = true;
+        wrote_colorspace = true;
     }
 
     // Write ICC profile, if we have anything
@@ -667,8 +690,10 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
             return "Could not set PNG iCCP chunk";
         unsigned char* icc_profile
             = (unsigned char*)icc_profile_parameter->data();
-        if (icc_profile && length)
+        if (icc_profile && length) {
             png_set_iCCP(sp, ip, "Embedded Profile", 0, icc_profile, length);
+            wrote_colorspace = true;
+        }
     }
 
     if (false && !spec.find_attribute("DateTime")) {
@@ -724,13 +749,17 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
     }
 
 #ifdef PNG_cICP_SUPPORTED
+    // Only automatically determine CICP from oiio::ColorSpace if we didn't
+    // write colorspace metadata yet.
     const ParamValue* p = spec.find_attribute(CICP_ATTR,
                                               TypeDesc(TypeDesc::INT, 4));
-    if (p) {
-        const int* int_vals = static_cast<const int*>(p->data());
+    cspan<int> cicp     = (p) ? p->as_cspan<int>()
+                          : (!wrote_colorspace) ? colorconfig.get_cicp(colorspace)
+                                                : cspan<int>();
+    if (!cicp.empty()) {
         png_byte vals[4];
         for (int i = 0; i < 4; ++i)
-            vals[i] = static_cast<png_byte>(int_vals[i]);
+            vals[i] = static_cast<png_byte>(cicp[i]);
         if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
             return "Could not set PNG cICP chunk";
         // libpng will only write the chunk if the third byte is 0
